#!/usr/bin/perl
#
# $Revision: 1.81 $
# Copyright 2009 Teleflora
# Re-purposed shell script for all remote teleflora customer service activities.
#

use strict;
use warnings;
use Getopt::Long;
use POSIX;
use Digest::MD5;


my $RTIDIR  = "";
my $VERSION = 0;
my $INSTALLKEYS = 0;
my $SUDOSETUP = 0;
my $HELP = 0;
my $key = "";


GetOptions(
	"installkeys" => \$INSTALLKEYS,
	"sudosetup" => \$SUDOSETUP,
	"version" => \$VERSION,
	"help" => \$HELP,
);


if($VERSION != 0) {
	print('$Revision: 1.81 $' . "\n");
	exit(0);
}

if($HELP != 0) {
	print("Usage:\n");
	print("$0\n");
	print("$0 --help\n");
	print("$0 --version\n");
	print("$0 --sudosetup\n");
	print("$0 --installkeys /path/to/authorized_keys\n");
	print("This script offers a system of menus for customer service to perform most\n");
	print("common tasks of an RTI server. This script does occasionally run commands via \n");
	print("'sudo', thus, a member of the POS 'admins' should run the script.\n");
	print("\n");
	exit(0);
}



$RTIDIR = $ENV{'RTI_DIR'};
if("$RTIDIR" eq "") {
	print("Warning: Assuming RTI is installed in /usr2/bbx\n");
	$RTIDIR="/usr2/bbx";
}



if($INSTALLKEYS != 0) {
	exit(install_keys(validate_input($ARGV[0])));
}

if($SUDOSETUP != 0) {
	exit(sudo_setup());
}


# Handle signals.
# Mainly, prevent "breaking out" to a shell.
$SIG{'STOP'} = 'IGNORE';
$SIG{'TSTP'} = 'IGNORE';



my $title = "";
my $command = "";
my $returnval =  "";
logevent("Starting Admin Menu");
while(1) {




	# Summary information about this shop.
	$title = titlestring("RTI Admin Menu");
	$title .= "(" . '$Revision: 1.81 $' . ")\\n";
	open(FILE, "< $RTIDIR/bbxd/RTI.ini");
	while(<FILE>) {
		chomp;
		if (/(VERSION)(\s*)(=)(\s*)([[:print:]]+)/) {
			$title .= "RTI Version: $5\\n";
			last;
		}
	}
	close(FILE);
	$title .= "\\n";

	$command = "dialog --stdout --timeout 900";
	$command .= " --no-cancel";
	$command .= " --default-item \"Exit\"";
	$command .= " --menu '$title'";
	$command .= " 0 0 0";
	$command .= " \"Backups\"" . " \"System and Data Backup Tools\"";
	if(is_admin_user() != 0) {
		$command .= " \"Commandline\"" . " \"Go the the Linux Shell\"";
	}
	$command .= " \"CC\"" . " \"Credit Card\"";
	$command .= " \"Dove\"" . " \"Dove\"";
	$command .= " \"Fax\"" . " \"Fax\"";
	$command .= " \"Network\"" . " \"Network Connectivity\"";
	$command .= " \"OS\"" . " \"Operating System\"";
	$command .= " \"Printers\"" . " \"Printers\"";
	$command .= " \"RTI\"" . " \"RTI Application\"";
	$command .= " \"Terminals\"" . " \"Terminals\"";
	$command .= " \"Users\"" . " \"Linux Users\"";
	$command .= " \"Exit\"" . " \"Exit\"";

	open(DIALOG, "$command |");
	$returnval = <DIALOG>;
	close(DIALOG);
	next if(! $returnval);
	next if("$returnval" eq "");
	chomp($returnval);




	if($returnval eq "timeout") {
		logevent("Inactivity Timeout");
		exit(0);

	# Backups.
	} elsif($returnval eq "Backups") {
		backup_menu();

	} elsif($returnval eq "CC") {
		cc_menu();

	# Prompt - Commandline.
	} elsif($returnval eq "Commandline") {
		logevent("Begin Shellout to Commandline.");
		system("PS1=\"\\n\\u\@\\h: \\w\\n\\d \\t \\\$ \" /bin/bash");
		logevent("Finish Shellout to Commandline.");


	# Dove
	} elsif($returnval eq "Dove") {
		dove_menu();

	# Fax
	} elsif($returnval eq "Fax") {
		fax_menu();


	} elsif($returnval eq "Network") {
		network_menu();

	} elsif($returnval eq "OS") {
		os_menu();

	} elsif($returnval eq "Printers") {
		printers_menu();


	} elsif($returnval eq "RTI") {
		rti_menu();

	} elsif($returnval eq "Terminals") {
		system("$RTIDIR/bin/killterms.pl");


	} elsif($returnval eq "Users") {
		users_menu_main();

	} elsif($returnval eq "Exit") {
		$returnval = menu_confirm("Exit RTI Admin Menu?", "No");
		if($returnval eq "Y") {
			logevent("Finish Admin Menu.");
			exit(0);
		}
	}
}

exit(0);
###############################################################################
###############################################################################
###############################################################################




sub install_keys
{
	my $keyfile = $_[0];
	my @ent = ();
	my $homedir = "";

	if( (! $keyfile) || ("$keyfile" eq "")) { 
		logevent("No authorized_keys file specified. No changes were made.");
		return(1);
	}
	if( (! -f "$keyfile") || (-s "$keyfile" == 0)) {
		logevent("File not found, or zero-byte file \"$keyfile\"");
		return(2);
	}



	# Where is tfsupport's home directory?
	$homedir = "";
	setpwent();
	@ent = getpwent();
	while(@ent) {
		if( "$ent[0]" eq "tfsupport") {
			$homedir = "$ent[7]";
			last;
		}
		@ent = getpwent();
	}
	endpwent();



	if("$homedir" eq "") {
		logevent("Could not find home directory for 'tfsupport' user.");
		return(1);
	}


	logevent("Installing SSH keys into \"$homedir\"");

	# Create .ssh directory, if need be.
	if(! -d "$homedir/.ssh") {
		logevent("Creating Directory \"$homedir/.ssh\"");
		mkdir("$homedir/.ssh");
		if(! -d "$homedir/.ssh") {
			logevent("Error: Could not create Directory \"$homedir/.ssh\"");
			return(3);
		}
		system("chown tfsupport:root $homedir/.ssh");
		system("chmod 740 $homedir/.ssh");
	}


	# Copy over our authorized_keys file.
	system("cp -v $keyfile $homedir/.ssh/authorized_keys");


	# Make sure we have something in place.
	if(! -f "$homedir/.ssh/authorized_keys") {
		logevent("Error: Could not create new 'authorized keys' file.");
		return(4);
	}

	# Setup permissions.
	system("chown tfsupport:root $homedir/.ssh/authorized_keys");
	system("chmod 700 $homedir/.ssh/authorized_keys");


	return(0);
}


sub users_menu_main
{
	my $title = "User Menu";
	my $command = "";
	my $returnval = "";
	my $suboption = "";
	my $user = "";

	while(1) {
		$title = titlestring("User Menu");

		$command = "dialog --stdout --timeout 900";
		$command .= " --no-cancel";
		$command .= " --default-item \"Close\"";
		$command .= " --menu '$title'";
		$command .= " 0 0 0";
		$command .= " \"Add User\"" . " \"Add User\"";
		$command .= " \"Add Admin\"" . " \"Add Administrator\"";
		$command .= " \"Delete\"" . " \"Delete User\"";
		$command .= " \"Info\"" . " \"Get User Info\"";
		$command .= " \"List\"" . " \"List all users.\"";
		$command .= " \"ResetPW\"" . " \"Reset Password\"";
		$command .= " \"Close\"" . " \"Close This Menu\"";

		open(USERDIALOG, "$command |");
		$returnval = <USERDIALOG>;
		close(USERDIALOG);
		next if(! $returnval);
		chomp($returnval);


		if($returnval eq "timeout") {
			logevent("Inactivity Timeout.");
			exit(0);
			
		} elsif($returnval =~ /(Add)(\s+)([[:print:]]+)/) {
			$user = menu_getstring("Choose a Username to Add");
			if($user ne "") {

				system("sudo $RTIDIR/bin/rtiuser.pl --info $user > /dev/null 2> /dev/null");
				if($? == 0) {
					menu_info("User \"$user\" already exists.\n");

				} else {

					if($returnval eq "Add Admin") {
						$suboption = menu_confirm("Add '$user' ADMIN Account?");
						if($suboption eq "Y") {
							system("sudo $RTIDIR/bin/rtiuser.pl --add $user");
							if($? == 0) {
								system("sudo $RTIDIR/bin/rtiuser.pl --enable-admin $user");
							}
						}
					} else {
						$suboption = menu_confirm("Add '$user' Account?");
						if($suboption eq "Y") {
							system("sudo $RTIDIR/bin/rtiuser.pl --add $user");
						}
					}
				}
			}



		} elsif($returnval eq "Info") {
			$user = users_menu_choose("Get User Information");
			if($user ne "") {
				system("sudo $RTIDIR/bin/rtiuser.pl --info $user 2>&1 | less");
			}

		} elsif($returnval eq "List") {
			system("sudo $RTIDIR/bin/rtiuser.pl --list 2>&1 | less");

		# Change Password
		} elsif($returnval eq "ResetPW") {
			$user = users_menu_choose("Which User to Reset Password?");
			if($user ne "") {
				$suboption = menu_confirm("Reset Password for '$user'?");
				if($suboption eq "Y") {
					system("sudo $RTIDIR/bin/rtiuser.pl --resetpw $user");
				}
			}


		# Delete User
		} elsif($returnval eq "Delete") {
			$user = users_menu_choose("Which User to Delete?");
			if($user ne "") {
				$suboption = menu_confirm("Delete User \"$user\"?");
				if($suboption eq "Y") {
					system("sudo $RTIDIR/bin/rtiuser.pl --delete $user 2>&1 | less");
				}
			}


		#  Exit
		} elsif($returnval eq "Close") {
			return(0);
		}
	}
}


sub backup_menu
{
	my $command = "";
	my $returnval = "";
	my $suboption = "";
	my $title = "Backup Menu";
	my $user = "";
	my $logfile = "";

	while(1) {

		$title = titlestring("Backup Menu");

		$command = "dialog --stdout --timeout 900";
		$command .= " --no-cancel";
		$command .= " --default-item \"Close\"";
		$command .= " --menu '$title'";
		$command .= " 0 0 0";
		$command .= " \"Backup Results\"" . " \"Display Backup Results\"";
		$command .= " \"Ltar Log\"" . " \"Lone-Tar \"ltar.log\" File\"";
		$command .= " \"Master Log\"" . " \"Lone-Tar \"Master\" Logfiles\"";
		$command .= " \"Eject\"" . " \"Eject Rev Tape\"";
		$command .= " \"LoneTar\"" . " \"Lone-Tar Menu\"";
		$command .= " \"Verify\"" . " \"Verify Tape Contents\"";
		if(-f "$RTIDIR/bin/mkudffs") {
			$command .= " \"Format\"" . " \"Re-Format Rev Cartridge\"";
		}
		$command .= " \"Close\"" . " \"Close This Menu\"";

		open(BACKDIALOG, "$command |");
		$returnval = <BACKDIALOG>;
		close(BACKDIALOG);
		next if(! $returnval);
		chomp($returnval);

		if($returnval eq  "timeout") {
			logevent("Inactivity Timeout.");
			exit(0);

		} elsif($returnval eq "Backup Results") {
			system("grep -B 3 \"RESULT:\" /log/ltar.log | tac | less");


		} elsif($returnval eq "Ltar Log") {
			viewfiles("/log/ltar.log");

		} elsif($returnval eq "Master Log") {
			viewfiles("/log/Master*");
			
		} elsif($returnval eq "Eject") {
			system("sudo /usr/sbin/eject /dev/scd0");

		} elsif($returnval eq "LoneTar") {
			system("sudo /usr/lone-tar/ltmenu");

		} elsif($returnval eq "Verify") {
			system("sudo /usr/lone-tar/tapetell");

		} elsif($returnval eq "Format") {
			system("sudo $RTIDIR/bin/mkudffs --udfrev=0x200 --media-type=dvdram --utf8 --vid=\"RTI Backup\" --lvid=\"RTI Backup\" /dev/scd0");

		} elsif($returnval eq "Close") {
			return(0);
		}
	}
}


sub cc_menu
{
	my $command = "";
	my $returnval = "";
	my $suboption = "";
	my $title = "Credit Card Menu\\n";
	my $user = "";
	my $logfile = "";



	while(1) {

		$title = titlestring("Credit Card Menu");

		# Which version of TCC?
		open(TCC, "$RTIDIR/bin/tcc --version |");
		while(<TCC>) {
			chomp;
			if(/Version:/) {
				$title .= "TCC $_\\n";
			}
		}
		close(TCC);

		# What is the md5sum of our tcc executable?
		open(TCC, "< $RTIDIR/bin/tcc");
			binmode(TCC);
			$title .= "TCC MD5: " . Digest::MD5->new->addfile(*TCC)->hexdigest . "\\n";
		close(TCC);




		$command = "dialog --stdout --timeout 900";
		$command .= " --no-cancel";
		$command .= " --default-item \"Close\"";
		$command .= " --menu '$title'";
		$command .= " 0 0 0";
		$command .= " \"Log\"" . " \"Show TCC Logfile\"";
		$command .= " \"Live Logs\"" . " \"Watch Live Logfile Activity\"";
		$command .= " \"Errors\"" . " \"TCC Errors and Warnings\"";
		$command .= " \"Close\"" . " \"Close This Menu\"";

		open(BACKDIALOG, "$command |");
		$returnval = <BACKDIALOG>;
		close(BACKDIALOG);
		next if(! $returnval);
		chomp($returnval);


		# Unlock a user.
		if($returnval eq  "timeout") {
			logevent("Inactivity Timeout.");
			exit(0);

		} elsif($returnval eq "Live Logs") {
			system("tail -f $RTIDIR/log/tcc.log");

		} elsif($returnval eq "Log") {
			viewfiles("$RTIDIR/log/tcc*.log");

		} elsif($returnval eq "Errors") {
			viewfiles("$RTIDIR/log/tcc*.log", "", "grep -e '<E>' -e '<W>'");

		} elsif($returnval eq "Close") {
			return(0);
		}
	}
}




sub dove_menu
{
	my $command = "";
	my $returnval = "";
	my $suboption = "";
	my $title = "";
	my $modemdev = "";
	my $thisday = strftime("%d", localtime());
	my $logfile = "";




	while(1) {


		$title = titlestring("Dove Menu");

		# TCC
		open(TCC, "$RTIDIR/bin/tcc --version |");
		while(<TCC>) {
			chomp;
			if(/Version:/) {
				$title .= "TCC $_\\n";
			}
		}
		close(TCC);

		# What version of doveserver are we running?
		open(VERS, "$RTIDIR/bin/doveserver.pl --version | ");
		while(<VERS>) {
			chomp;
			next until(/(Revision:\s)([[:print:]]+)(\$)/);
			$title .= "Doveserver Revision $2\\n";
			last;
		}
		close(VERS);

		# Modems
		open(CONFIG, "< $RTIDIR/config/dove.ini");
		while(<CONFIG>) {
			chomp;
			if(/^(\s*)(DOVE_LISTEN_MODEM)(\s*)(=)(\s*)([[:print:]]+)/) {
				$title .= "Inbound Modem: $6\\n";
			}
			if(/^(\s*)(DOVE_DIALOUT_MODEM_1)(\s*)(=)(\s*)([[:print:]]+)/) {
				$title .= "Outbound Modem: $6\\n";
			}
			if(/^(\s*)(DOVE_USERNAME)(\s*)(=)(\s*)([[:print:]]+)/) {
				$title .= "Shopcode: $6\\n";
			}
		}
		close(CONFIG);
		$title .= "\\n";



		$command = "dialog --stdout --timeout 900";
		$command .= " --no-cancel";
		$command .= " --default-item \"Close\"";
		$command .= " --menu '$title'";
		$command .= " 0 0 0";
		$command .= " \"Callout Errors\"" . " \"Callout Errors and Warnings\"";
		$command .= " \"Callout Log\"" . " \"Callout Logfile\"";
		$command .= " \"Doveserver Errors\"" . " \"Doveserver Errors and Warnings\"";
		$command .= " \"Doveserver Log\"" . " \"Doveserver Logfile\"";
		$command .= " \"Live Logs\"" . " \"Watch Live Logfile Activity\"";
		$command .= " \"Stop\"" . " \"Stop Doveserver\"";
		$command .= " \"Start\"" . " \"Start Doveserver\"";
		$command .= " \"Status\"" . " \"Doveserver Status\"";
		$command .= " \"Close\"" . " \"Close This Menu\"";

		open(DOVEDIALOG, "$command |");
		$returnval = <DOVEDIALOG>;
		close(DOVEDIALOG);
		next if(! $returnval);
		chomp($returnval);


		# Callout logfile.
		if($returnval eq  "timeout") {
			logevent("Inactivity Timeout.");
			exit(0);

		} elsif($returnval eq "Live Logs") {
			system("tail -f $RTIDIR/log/doveserver-Day_$thisday.log $RTIDIR/log/callout-Day_$thisday.log");

		} elsif($returnval eq "Callout Errors") {
			viewfiles("$RTIDIR/log/callout-*.log", "", "grep -e '<E>' -e '<W>'");

		} elsif($returnval eq "Callout Log") {
			viewfiles("$RTIDIR/log/callout-*.log");

		} elsif($returnval eq "Doveserver Errors") {
			viewfiles("$RTIDIR/log/doveserver-*.log", "", "grep -e '<E>' -e '<W>'");

		} elsif($returnval eq "Doveserver Log") {
			viewfiles("$RTIDIR/log/doveserver-*.log");

		} elsif($returnval eq "Stop") {
			system("(sudo $RTIDIR/bin/doveserver.pl --stop) | less");

		} elsif($returnval eq "Start") {
			system("(sudo $RTIDIR/bin/doveserver.pl --start) | less");

		} elsif($returnval eq "Status") {
			system("watch --interval=10 \"$RTIDIR/bin/doveserver.pl --status\" ");

		} elsif($returnval eq "Close") {
			return(0);
		}
	}
}





sub fax_menu
{
	my $command = "";
	my $returnval = "";
	my $thisday = strftime("%d", localtime());
	my $title = "Fax Menu\\n";
	my $logfile = "";



	while(1) {


		$title = titlestring("Fax Menu");

		# What version of fax are we running?
		open(VERS, "$RTIDIR/bin/rti_sendfax.pl --version | ");
		while(<VERS>) {
			chomp;
			next until(/(Revision:\s)([[:print:]]+)(\$)/);
			$title .= "rti_sendfax.pl Revision $2\\n";
			last;
		}
		close(VERS);



		# Modems
		open(CONFIG, "< $RTIDIR/config/fax.ini");
		while(<CONFIG>) {
			chomp;
			if(/^(\s*)(FAX_MODEM_1)(\s*)(=)(\s*)([[:print:]]+)/) {
				$title .= "Faxout Modem: $6\\n";
			}
		}
		close(CONFIG);
		$title .= "\\n";



		$command = "dialog --stdout --timeout 900";
		$command .= " --no-cancel";
		$command .= " --default-item \"Close\"";
		$command .= " --menu '$title'";
		$command .= " 0 0 0";
		$command .= " \"Log\"" . " \"Show Fax Log\"";
		$command .= " \"Live Logs\"" . " \"Watch Live Logfile Activity\"";
		$command .= " \"Queue\"" . " \"Queue - Show Fax Queue\"";
		$command .= " \"Statistics\"" . " \"Call Statistics\"";
		$command .= " \"Unlock\"" . " \"Unlock Fax Process\"";
		$command .= " \"Close\"" . " \"Close This Menu\"";

		open(FAXDIALOG, "$command |");
		$returnval = <FAXDIALOG>;
		close(FAXDIALOG);
		next if(! $returnval);
		chomp($returnval);


		if($returnval eq  "timeout") {
			logevent("Inactivity Timeout.");
			exit(0);

		} elsif($returnval eq "Log") {
			viewfiles("$RTIDIR/log/rti_sendfax-*.log");

		} elsif($returnval eq "Live Logs") {
			system("tail -f $RTIDIR/log/rti_sendfax-Day_$thisday.log");

		} elsif($returnval eq "Unlock") {
			system("$RTIDIR/bin/rti_sendfax.pl --unlock");

		} elsif($returnval eq "Queue") {
			system("watch --interval=10 \"$RTIDIR/bin/rti_sendfax.pl --list\"");

		} elsif($returnval eq "Statistics") {
			system("grep 'dialing' $RTIDIR/log/rti_sendfax-*.log | wc -l > /tmp/report.$$");
			system("grep 'Error:' $RTIDIR/log/rti_sendfax-*.log | awk '{print \$3,\$4,\$5,\$6,\$7}' | sort | uniq -c | sort -r >> /tmp/report.$$");
			system("less /tmp/report.$$ && rm -f /tmp/report.$$");

		} elsif($returnval eq "Close") {
			return(0);
		}
	}
}


sub network_menu
{
	my $command = "";
	my $returnval = "";
	my $suboption = "";
	my $ipaddr = "";
	my $macaddr = "";
	my $title = "Network Menu\n";
	my $user = "";
	my $device = "";
	my $logfile = "";
	my $i = 0;
	my $temp = "";




	$title .="\n";
	while(1) {

		$title = titlestring("Network Menu");

		# Hostname
		open(PIPE, "hostname --long |");
		$returnval = <PIPE>;
		close(PIPE);
		chomp($returnval);
		$title .= "Hostname: $returnval\\n";

		# Private IP Address(es)
		foreach $device("eth0", "eth1", "bond0") {
			$ipaddr = "";
			$macaddr = "";
			open(PIPE, "/sbin/ifconfig $device |");
			while(<PIPE>) {
				chomp;
				if(/(inet addr:)(\d+\.\d+\.\d+\.\d+)/) {
					$ipaddr = $2;
					$ipaddr =~ s/[[:space:]]+//g;
				}
				if(/(HWaddr)(\s+)([[:print:]]+)/) {
					$macaddr = $3;
					$macaddr =~ s/[[:space:]]+//g;
				}
				
			}
			close(PIPE);
			if("$ipaddr" ne "") {
				$title .= "$device IPADDR=\"$ipaddr\" MAC=\"$macaddr\" \\n";
			}
		}


		$command = "dialog --stdout --timeout 900";
		$command .= " --no-cancel";
		$command .= " --default-item \"Close\"";
		$command .= " --menu '$title'";
		$command .= " 0 0 0";
		$command .= " \"Configure\"" . " \"Configure Ethernet Device(s)\"";
		$command .= " \"Arp\"" . " \"View Current Arp Cache\"";
		$command .= " \"Elinks\"" . " \"'elinks' text based web browser\"";
		$command .= " \"DNS\"" . " \"Domain Name Service (DNS)\"";
		$command .= " \"Samba\"" . " \"Samba Share Status\"";
		$command .= " \"Pings\"" . " \"Ping Various Hosts\"";
		for($i = 1; $i <= 20; $i++) {
			open(HOSTS, "< /etc/hosts.allow");
			while(<HOSTS>) {
				if(/^(\s*)(sshd:)(\s*)(192\.168\.$i)/) {
					$command .= " \"Network Scan .$i\"" . " \"Scan 192.168.$i Network for Hosts\"";
					last;
				}
			}
			close(HOSTS);
		}

		$command .= " \"Ifconfig\"" . " \"Network Interface Information.\"";
		$command .= " \"Routes\"" . " \"Routing Information.\"";
		$command .= " \"Close\"" . " \"Close This Menu\"";

		open(BACKDIALOG, "$command |");
		$returnval = <BACKDIALOG>;
		close(BACKDIALOG);
		next if(! $returnval);
		chomp($returnval);


		if($returnval eq  "timeout") {
			logevent("Inactivity Timeout.");
			exit(0);

		} elsif($returnval eq "Arp") {
			system("watch --interval=10 \"/sbin/arp -vn\"");

		} elsif($returnval eq "Configure") {
			system("sudo /usr/sbin/system-config-network");

		} elsif($returnval eq "Elinks") {
			system("elinks");

		} elsif($returnval eq "DNS") {
			$command = "cat /etc/resolv.conf";
			$command .= " ; /bin/echo ; /bin/echo";
			$command .= " ; nslookup www.teleflora.com";
			$command .= " ; /bin/echo ; /bin/echo";
			$command .= " ; nslookup igusproda.globalpay.com";
			system("($command) | less");

		} elsif($returnval eq "Samba") {
			system("watch --interval=10 \"smbstatus\"");

		} elsif($returnval eq "Ifconfig") {
			system("watch --interval=10 \"ifconfig\"");



		} elsif($returnval eq "Pings") {
			$command = "ping -c 5 4.2.2.2";
			$command .= " ; /bin/echo ; /bin/echo";
			$command .= " ; ping -c 5 igusproda.globalpay.com";
			$command .= " ; /bin/echo ; /bin/echo";
			$command .= " ; ping -c 5 igusprodb.globalpay.com";
			$command .= " ; /bin/echo ; /bin/echo";
			$command .= " ; ping -c 5 xmlrpc.rhn.redhat.com";
			system("($command) | less");


		} elsif($returnval eq "Routes") {
			$command = "/sbin/ifconfig";
			$command .= " ; /bin/echo ; /bin/echo";
			$command .= " ; /sbin/route -n";
			$command .= " ; /bin/echo ; /bin/echo";
			$command .= " ; traceroute www.teleflora.com";
			system("($command) | less");
			

		} elsif($returnval eq "Close") {
			return(0);
		}

		# Network Scan .x
		for($i = 1; $i <= 20; $i++) {
			if($returnval eq "Network Scan .$i") {
				system("(sudo /usr/bin/nmap -sS -O 192.168.$i.1/16) | less");
				last;
			}
		}
	}
}

sub os_menu
{
	my $command = "";
	my $returnval = "";
	my $suboption = "";
	my $title = "Operating System (OS) Menu";
	my $user = "";
	my $runlevel = "";
	my $logfile = "";

	while(1) {

		$title = titlestring("Operating System (OS) Menu");


		# Which version of Linux?
		open(FILE, "< /etc/redhat-release");
		while(<FILE>) {
			chomp;
			if(/Linux/) {
				$title .= "$_\\n";
			}
		}
		close(FILE);


		# Which version of Linux?
		open(UNAME, "uname --kernel-release --processor |");
		while(<UNAME>) {
			chomp;
			if("$_" ne "") {
				$title .= "Kernel: $_\\n";
			}
		}
		close(UNAME);

				

		$command = "dialog --stdout --timeout 900";
		$command .= " --no-cancel";
		$command .= " --default-item \"Close\"";
		$command .= " --menu '$title'";
		$command .= " 0 0 0";
		$command .= " \"dmesg\"" . " \"Recent Kernel Messages\"";
		$command .= " \"Disk\"" . " \"Disk Usage Statistics\"";
		$command .= " \"DOSDir Floppy\"" . " \"Current Floppy Disk Contents\"";
		$command .= " \"Halt\"" . " \"Halt the Server\"";
		$command .= " \"Live Logs\"" . " \"Watch Live Logfile Activity\"";
		$command .= " \"Messages\"" . " \"/var/log/messages Logs.\"";
		$command .= " \"Minicom\"" . " \"Modem Terminal Program.\"";
		$command .= " \"PS\"" . " \"Current Running Processes (PS)\"";
		$command .= " \"Reboot\"" . " \"Reboot the Server.\"";
		$command .= " \"Redhat SOS\"" . " \"Create a Redhat 'SOS' Report.\"";
		$command .= " \"Redhat List\"" . " \"List Available Redhat Updates\"";
		$command .= " \"Redhat Updates\"" . " \"Apply Redhat OS Updates\"";
		$command .= " \"Secure\"" . " \"/var/log/secure Logs.\"";
		$command .= " \"Server\"" . " \"Get Server Information\"";
		$command .= " \"Services\"" . " \"Show Running Services\"";
		$command .= " \"Uptime\"" . " \"Reboot History\"";
		$command .= " \"Top\"" . " \"Running Processes (Top)\"";
		$command .= " \"Who\"" . " \"Who is currently Logged In\"";
		$command .= " \"vmstat\"" . " \"CPU and Swap Status\"";
		if(-f "/usr/bin/mpi") {
			$command .= " \"Digi\"" . " \"Run Digi Control Software\"";
		}
		$command .= " \"Close\"" . " \"Close This Menu\"";

		open(OSDIALOG, "$command |");
		$returnval = <OSDIALOG>;
		close(OSDIALOG);
		next if(! $returnval);
		chomp($returnval);


		if($returnval eq  "timeout") {
			logevent("Inactivity Timeout.");
			exit(0);

		} elsif($returnval eq "dmesg") {
			system("dmesg | less");

		} elsif($returnval eq "Disk") {
			system("watch --interval=10 \"df -h\"");
		} elsif($returnval eq "DOSDir Floppy") {
			system("sudo dosdir A: | less");

		} elsif($returnval eq "Digi") {
			system("sudo /usr/bin/mpi");

		} elsif($returnval eq "Halt") {
			$returnval = menu_confirm("Halt the RTI Server?");
			if($returnval eq "Y") {
				$returnval = menu_confirm("Five-Minute Warning? (Halt)");
				if($returnval eq "Y") {
					system("sudo /sbin/shutdown -h \"+5\"");
				} else {
					system("sudo /sbin/shutdown -h now");
				}
			}



		} elsif($returnval eq "Live Logs") {
			system("sudo tail -f /var/log/messages /var/log/secure");
		} elsif($returnval eq "Redhat SOS") {
			system("sudo /usr/sbin/sosreport");



		} elsif($returnval eq "Reboot") {
			$returnval = menu_confirm("Reboot the RTI Server?");
			if($returnval eq "Y") {
				$returnval = menu_confirm("Five-Minute Warning? (Reboot)");
				if($returnval eq "Y") {
					system("sudo /sbin/shutdown -r \"+5\"");
				} else {
					system("sudo /sbin/shutdown -r now");
				}
			}


		} elsif($returnval eq "Redhat List") {
			# RHWS 5
			if(-f "/usr/bin/yum") {
				system("sudo yum clean all && sudo /usr/bin/yum check-update 2>&1 | less");
			# RHEL 4
			} elsif (-f "/usr/sbin/up2date") {
				system("sudo /usr/sbin/up2date --list 2>&1 | less");
			}


		} elsif($returnval eq "Redhat Updates") {
			$returnval = menu_confirm("Download and Apply Redhat OS Updates Now?");
			if($returnval eq "Y") {



				if(-f "/usr/bin/yum") {
					$command = "sudo /usr/bin/yum clean all && sudo /usr/bin/yum clean all";
					$command .= " ; sudo /usr/bin/yum update -y ";
				} elsif (-f "/usr/sbin/up2date") {
					$command = "sudo /usr/sbin/up2date -u";
				}



				system("$command");
				system("sudo /usr/sbin/aide --init");
				system("sudo cp -u /var/lib/aide/aide.db.new.gz /var/lib/aide/aide.db.gz");
			}


		} elsif($returnval eq "Server") {
			$command = "sudo /usr/sbin/dmidecode | grep -A 5 \"System Information\"";
			$command .= " ; sudo /usr/sbin/dmidecode | grep -A 4 \"BIOS Information\"";
			$command .= " ; /bin/echo ; /bin/echo";
			$command .= " ; cat /etc/redhat-release";
			$command .= " ; uname --kernel-release --processor";
			$command .= " ; /bin/echo ; /bin/echo";
			$command .= " ; cat /proc/meminfo";
			$command .= " ; /bin/echo ; /bin/echo";
			$command .= " ; /sbin/lsusb";
			$command .= " ; /bin/echo ; /bin/echo";
			$command .= " ; /sbin/lspci";
			$command .= " ; /bin/echo ; /bin/echo";
			$command .= " ; /sbin/lsmod";
			$command .= " ; /bin/echo ; /bin/echo";
			$command .= " ; cat /proc/interrupts";
			system("($command) | less");


		} elsif($returnval eq "Services") {

			open(INIT, "< /etc/inittab");
			while(<INIT>) {
				if(/^(\s*)(id:)(\d+)(:)/) {
					$runlevel = $3;
					last;
				}
			}
			close(INIT);
			system("/sbin/chkconfig --list | grep -v $runlevel:off | less");

		

		} elsif($returnval eq "Messages") {
			viewfiles("/var/log/messages*", "sudo");

		} elsif($returnval eq "Minicom") {
			system("minicom");

		} elsif($returnval eq "PS") {
			system("watch --interval=5 \"ps wwaux\"");
			
		} elsif($returnval eq "Secure") {
			viewfiles("/var/log/secure*", "sudo");

		} elsif($returnval eq "Uptime") {
			$command = "last reboot";
			$command .= " ; /bin/echo ; /bin/echo";
			$command .= " ; uptime";
			system("($command) | less");



		} elsif($returnval eq "vmstat") {
			system("vmstat 2 450");

		} elsif($returnval eq "Who") {
			system("watch /usr/bin/w");
			
		} elsif($returnval eq "Top") {
			system("sudo top");

		} elsif($returnval eq "Close") {
			return(0);
		}
	}
}



sub printers_menu
{
	my $command = "";
	my $returnval = "";
	my $suboption = "";
	my $title = "";
	my $user = "";
	my $logfile = "";

	while(1) {
		$title = titlestring("Printer Menu");
		$title .= "\n\n";
		$title .= get_printer_status();

		$command = "dialog --stdout --timeout 900";
		$command .= " --no-cancel";
		$command .= " --default-item \"Close\"";
		$command .= " --menu '$title'";
		$command .= " 0 0 0";
		$command .= " \"ConfigBBX\"" . " \"Config.bbx\"";
		$command .= " \"Live Logs\"" . " \"Watch Live Logfile Activity\"";
		$command .= " \"Redhat Config\"" . " \"Redhat Printer Config\"";
		$command .= " \"Error\"" . " \"View Cups Error Logfile\"";
		$command .= " \"Kill\"" . " \"Kill a Print Job\"";
		$command .= " \"Restart\"" . " \"Restart Cups\"";
		$command .= " \"Queues\"" . " \"View Printer Queues\"";
		$command .= " \"Close\"" . " \"Close SubMenu\"";

		open(BACKDIALOG, "$command |");
		$returnval = <BACKDIALOG>;
		close(BACKDIALOG);
		next if(! $returnval);
		chomp($returnval);


		if($returnval eq "Redhat Config") {
			system("sudo /usr/sbin/system-config-printer");

		} elsif($returnval eq "ConfigBBX") {
			viewfiles("$RTIDIR/config/config.bbx");
		
		} elsif($returnval eq "Error") {
			viewfiles("/var/spool/cups/error_log*", "sudo");

		} elsif($returnval eq "Live Logs") {
			system("sudo tail -f /var/log/cups/error_log /var/log/cups/access_log /var/log/cups/page_log");

		} elsif($returnval eq "Kill") {
			$suboption = menu_getstring("Which Print Job to Kill?");
			if($suboption ne "") {
				system("sudo cancel $suboption");
				menu_info("Print Job \"$suboption\" Killed.");
			}

		} elsif($returnval eq "Restart") {
			system("sudo /sbin/service cups restart");

		} elsif($returnval eq "Queues") {
			show_printer_queues();

		} elsif($returnval eq "Close") {
			return(0);
		}
	}
}



sub get_printer_status
{
	my @array = ();
	my $printer = "";
	my $found = 0;
	my $message = "";
	my $timestamp = "";


	# Is cups service running?
	$found = 0;
	open(PIPE, "sudo /sbin/service cups status |");
	while(<PIPE>) {
		if(/is running/) {
			$found =  1;
			last;
		}
	}
	close(PIPE);
	if($found == 0) {
		$message .= "Cups System Service is NOT running.\\n";
	} else {
		$message .= "Cups System Service is RUNNING.\\n";
	}




	# Is Cups listening on TCP Port?
	open(PIPE, "sudo netstat -plan |");
	while(<PIPE>) {
		if(/(127\.0\.0\.1:631)([[:print:]]+)(LISTEN)/) {
			$found = 1;
			last;
		}
	}
	close(PIPE);
	if($found == 0) {
		$message .= "Cups Printer Daemon is NOT LISTENING on TCP port.\\n";
	}


	# Cups Logfiles.
	$found = 0;
	open(FILE, "< /var/log/cups/error_log");
	while(<FILE>) {
		if(/^E/) {
			$found = 1;
			last;
		}
	}
	close(FILE);
	if($found != 0) {
		$message .= "There are Cups LOGFILE ERRORS.\\n";
	} else {
		$message .= "Cups logfiles are CLEAN.\\n";
	}

	# system("sudo cat /var/log/cups/error_log | less");
	# system("sudo cat /var/log/cups/access_log | less");
	# system("sudo cat /var/log/cups/page_log | less");

	# How many files in /var/spool/cups/tmp? 
	$found = 0;
	open(PIPE, "sudo /usr/bin/find /var/spool/cups/tmp -type f -print |");
	while(<PIPE>) {
		$found++;
	}
	close(PIPE);
	if($found > 1000) {
		$message .= "$found jobs in /var/spool/cups/tmp\\n";
	}

	return($message);
}

#
# Show "lpq" for each and every printer.
#
sub show_printer_queues
{
	my @array = ();
	my @printers = ();
	my $printer = "";
	my $command = "";
	my $i = 0;


	# Which printers are present?
	# What are their queue counts.
	open(PRINTERS, "lpstat -a |");
	while(<PRINTERS>) {
		@array = split(/[[:space:]]+/);
		$printer = $array[0];
		next if("$printer" eq "");
		push(@printers, "$printer");
	}
	close(PRINTERS);

	# We did not find a printer. This is odd.
	if($#printers < 0) {
		system("echo \"No Printers Found.\" | less");
		return(0);
	}

	$command = "(date ";
	$command .= " ; sudo lpq -P $printers[0]";
	for ($i = 1; $i <= $#printers; $i++) {
		$command .= " ; /bin/echo";
		$command .= " ; /bin/echo";
		$command .= " ; sudo lpq -P $printers[$i]";
	}
	$command .= ") | less";

	system("$command");

	return(0);
}


#
# Request a job ID, and then kill that.
#
sub kill_printjob
{
	my $command = "";
	my $returnval = "";

	$command = "dialog --stdout --timeout 900";
	$command .= " --no-cancel";
	$command .= " --inputbox 'Enter a Cups Job ID to Cancel'";
	$command .= " 0 0";


	open(DIALOG, "$command |");
	$returnval = <DIALOG>;
	close(DIALOG);
	next if(! $returnval);
	chomp($returnval);

	if($returnval eq "timeout") {
		logevent("Inactivity Timeout.");
		exit(0);
	}

	if("$returnval " eq "") {
		menu_info("No Print Job Specified.");
		return(0);
	}

	$returnval = validate_input($returnval);
	if("$returnval" ne "") {
		system("sudo cancel $returnval");
		menu_info("Print Job \"$returnval\" Cancelled.");
	}
}


# Pick a user from the results of "rtiuser.pl".
sub users_menu_choose
{
	my $title = $_[0];
	my $command = "";
	my $returnval = "";
	my @array = ();

	$command = "dialog --stdout --timeout 900";
	$command .= " --menu '$title'";
	$command .= " 0 0 0";

	# Create a list of all users who could be unlocked.
	open(PIPE, "sudo $RTIDIR/bin/rtiuser.pl --list | grep User | sort |");
	while(<PIPE>) {
		chomp;
		@array = split(/[[:space:]]+/);
		$command .= " \"$array[0]\" \"$array[1] $array[2] $array[3] $array[4]\"";
	}
	close(PIPE);

	open(USERCHOOSE, "$command |");
	$returnval = <USERCHOOSE>;
	close(USERCHOOSE);
	next if(! $returnval);
	chomp($returnval);


	if($returnval eq "timeout") {
		logevent("Inactivity Timeout.");
		exit(0);
	}


	return($returnval);
}



sub rti_menu
{
	my $command = "";
	my $returnval = "";
	my $suboption = "";
	my $title = "RTI Application Menu";
	my $user = "";
	my $logfile = "";

	while(1) {

		$title = titlestring("RTI Application Menu");

		$command = "dialog --stdout --timeout 900";
		$command .= " --no-cancel";
		$command .= " --default-item \"Close\"";
		$command .= " --menu '$title'";
		$command .= " 0 0 0";
		$command .= " \"RTI as Admin\"" . " \"Run RTI as tfsupport\"";
		$command .= " \"RTI as User\"" . " \"Run RTI as another user.\"";
		$command .= " \"Altiris Menu\"" . " \"Run Altiris Updates Menu\"";
		$command .= " \"Altiris Checkin\"" . " \"Perform an Altiris 'Checkin'\"";
		$command .= " \"Basis Logs\"" . " \"Basis License Manager Logs.\"";
		$command .= " \"Killem\"" . " \"Stop RTI Background Processes.\"";
		$command .= " \"Startbbx\"" . " \"Restart RTI Background Processes.\"";
		$command .= " \"Checkit\"" . " \"RTI Background Process Status.\"";
		$command .= " \"Patchlog\"" . " \"View RTI Patch Logfile.\"";
		$command .= " \"Close\"" . " \"Close This Menu\"";

		open(BACKDIALOG, "$command |");
		$returnval = <BACKDIALOG>;
		close(BACKDIALOG);
		next if(! $returnval);
		chomp($returnval);


		if($returnval eq "timeout") {
			logevent("Inactivity Timeout.");
			exit(0);

		} elsif($returnval eq "Altiris Checkin") {
			system("sudo $RTIDIR/bin/altiriscontrol.pl --checkin | less");


		} elsif($returnval eq "Altiris Menu") {
			system("sudo $RTIDIR/bin/altiriscontrol.pl --updates");


		} elsif($returnval eq "RTI as Admin") {
			system("sudo -u tfsupport $RTIDIR/bin/rti");

		} elsif($returnval eq "RTI as User") {
			$returnval = users_menu_choose("Choose RTI User");
			if($returnval ne "") {
				system("sudo -u $returnval $RTIDIR/bin/rti");
			}


		} elsif($returnval eq "Basis Logs") {
			viewfiles("/usr2/basis/blmgr/log/blm.log*");

		} elsif($returnval eq "Checkit") {
			system("watch --interval=10 \"$RTIDIR/bin/checkit\"");


			
		} elsif($returnval eq "Killem") {
			system("(sudo $RTIDIR/bin/killem ; checkit) | less");

		} elsif($returnval eq "Patchlog") {
			viewfiles("$RTIDIR/log/RTI-Patches.log");



		} elsif($returnval eq "Startbbx") {
			system("(sudo $RTIDIR/bin/startbbx ; checkit) | less");


		} elsif($returnval eq "Close") {
			return(0);
		}
	}
}






sub menu_confirm
{
	my $title = $_[0];
	my $default = $_[1];
	my $command = "";


	$command = "dialog --stdout --timeout 900";
	if( ($default) && ($default =~ /^no$/i)) {
		$command .= " --defaultno";
	}
	$command .= " --yesno";
	$command .= " \"$title\"";
	$command .= " 0 0";

	system("$command");

	if($? == 0) {
		return("Y");

	} elsif ($? == 255) {
		# Timeout
		logevent("Inactivity Timeout.");
		exit(0);
		
	} else {
		return("N");
	}
}


sub menu_info
{
	my $title = $_[0];
	my $command = "";


	$command = "dialog --stdout --timeout 900 --msgbox";
	$command .= " \"$title\"";
	$command .= " 0 0";

	system("$command");
	if($? == 255) {
		# Timeout
		logevent("Inactivity Timeout.");
		exit(0);
	}

	return(0);
}


sub menu_getstring
{
	my $title = $_[0];
	my $command = "";
	my $returnval = "";

	$command = "dialog --stdout --timeout 900";
	$command .= " --no-cancel";
	$command .= " --inputbox '$title'";
	$command .= " 0 0";


	open(DIALOG, "$command |");
	$returnval = <DIALOG>;
	close(DIALOG);
	next if(! $returnval);
	chomp($returnval);
	
	$returnval = validate_input($returnval);

	if($returnval eq "timeout") {
		# Timeout
		logevent("Inactivity Timeout.");
		exit(0);
	}

	return("$returnval");
}


sub titlestring
{
	my $title = $_[0];
	my $returnval = "";

	$returnval = "--------======== $title =======---------\\n";
	
	# Get our hostname.
	open(HOST, "/bin/hostname --short |");
	while(<HOST>) {
		chomp;
		if("$_" ne "") {
			$returnval .= "$_";
			last;
		}
	}
	close(HOST);

	# Current timestamp.
	$returnval .= " - ";
	$returnval .= strftime("%a %B %d, %Y %T %Z", localtime());
	$returnval .= "\\n";
	$returnval .= "\\n";



	return($returnval);
}




sub viewfiles
{
	my $filepattern = $_[0];
	my $sudo = $_[1];
	my $filter = $_[2];
	my @files = ();
	my $file = "";
	my $lasttype = "";
	my $command = "";


	if((! $filepattern) || ("$filepattern" eq "")) {
		return("");
	}


	if (($sudo) && ("$sudo" ne "")) {
		$sudo = "sudo ";
	} else {
		$sudo = "";
	}

	if(! $filter) {
		$filter = "";
	}


	if("$sudo" ne "") {
		open(FILELIST, "sudo /bin/ls -t $filepattern |");
	} else {
		open(FILELIST, "/bin/ls -t $filepattern |");
	}
	while(<FILELIST>) {
		chomp;
		push(@files, $_);
	}
	close(FILELIST);





	$command = "(/bin/true";
	$lasttype = "";
	foreach $file(@files) {

		if($file =~ /\.gz$/) {
			$command .= " ; $sudo /bin/zcat $file | tac";
		} else {
			$command .= " ; $sudo /usr/bin/tac $file";
		}
	}
	$command .= ") ";
	if($filter ne "") {
		$command .= " | $filter";
	}
	$command .= " | less";
	system("$command");


	return($command);
}




# PCI 6.5.6
# "some string; `cat /etc/passwd`" -> "some string cat /etc/passwd"
# "`echo $ENV; $(sudo ls)`" -> "echo ENV sudo ls"
# etc.
sub validate_input
{
	my $var = $_[0];
	my $temp = "";

	if(! $var) {
		return "";
	}

	$temp = $var;

	# "`bad command`" -> "bad command"
	$temp =~ s/\`//g;

	# "$(bad command)" -> "bad command"
	$temp =~ s/(\$\()(.*.)(\))/$2/g;


	# "stuff ; bad command" -> "stuff bad command"
	$temp =~ s/\;//g;

	# "stuff && bad command" -> "stuff bad command"
	$temp =~ s/\&//g;

	# "stuff | bad command" -> "stuff bad command"
	$temp =~ s/\|//g;

	# "stuff > bad command" -> "stuff bad command"
	$temp =~ s/\>//g;

	# "stuff < bad command" -> "stuff bad command"
	$temp =~ s/\<//g;

	# Filter non printables
	$temp =~ s/[[:cntrl:]]//g;


	return($temp);
}


# 
# Is the current user an administrative user?
#
sub is_admin_user
{
	my $returnval = 0;
	my @array = ();


	# We can have a shll option if we are either tfsupport, or root.
	# In 12.6 systems, "tfsupport" and "root" are always memebers of rtiadmins, however,
	# in 12.5 systmes, such may not be the case.
	open(NAME, "/usr/bin/whoami |");
	while(<NAME>) {
		chomp;
		if(/^tfsupport$/) {
			$returnval = 1;
		} elsif(/^root$/) {
			$returnval = 1;
		}
	}
	close(NAME);
	if($returnval != 0) {
		return($returnval);
	}


	open(GROUPS, "groups |");
	while(<GROUPS>) {
		chomp;
		@array = split(/\s+/);
		if(grep(/^rtiadmins$/, @array)) {
			$returnval = 1;
		}
	}
	close(GROUPS);

	return($returnval);
}


#
# Send an event to syslog.
#
sub logevent
{
	my $event = $_[0];
	my $logger = "";

	if(-f "/bin/logger") {
		$logger = "/bin/logger";
	} elsif(-f "/usr/bin/logger") {
		$logger = "/usr/bin/logger";
	} else {
		# Hope for the best.
		$logger = "logger";
	}

	system("$logger \"(PID $$: $ENV{'USER'}\@$ENV{'SSH_CLIENT'}) $event\"");
	print("$event\n");
}


#
# Setup sudo rules which will make these menus work much nicer.
#
sub sudo_setup
{

	# Our rules are already in place.
	system("grep \"dmidecode\" /etc/sudoers > /dev/null 2> /dev/null");
	if($? == 0) {
		return(0);
	}


	system("cp /etc/sudoers /etc/sudoers.$$");
	if(! -f "/etc/sudoers.$$") {
		return(1);
	}
	if(-s "/etc/sudoers.$$" <= 0) {
		return(2);
	}

	open(OLD, "< /etc/sudoers.$$");
	open(NEW, "> /etc/sudoers");
	while(<OLD>) {
		print(NEW);

		# Additional sudo entries related to our admin menu.
		if(/^(\s*)(tfsupport)(\s+)(ALL)(\s*)(=)(\s*)(\(ALL\))(\s*)(ALL)/) {
			print(NEW "tfsupport ALL=NOPASSWD: /bin/netstat\n");
			print(NEW "tfsupport ALL=NOPASSWD: /bin/true\n");
			print(NEW "tfsupport ALL=NOPASSWD: /bin/ls\n");
			print(NEW "tfsupport ALL=NOPASSWD: /bin/zcat\n");
			print(NEW "tfsupport ALL=NOPASSWD: /usr/lone-tar/ltmenu\n");
			print(NEW "tfsupport ALL=NOPASSWD: /sbin/service\n");
			print(NEW "tfsupport ALL=NOPASSWD: /sbin/shutdown\n");
			print(NEW "tfsupport ALL=NOPASSWD: /usr/bin/find\n");
			print(NEW "tfsupport ALL=NOPASSWD: /usr/bin/top\n");
			print(NEW "tfsupport ALL=NOPASSWD: /usr/bin/mpi\n");
			print(NEW "tfsupport ALL=NOPASSWD: /usr/bin/yum\n");
			print(NEW "tfsupport ALL=NOPASSWD: /usr/bin/cancel\n");
			print(NEW "tfsupport ALL=NOPASSWD: /usr/bin/lpq\n");
			print(NEW "tfsupport ALL=NOPASSWD: /usr/bin/tail\n");
			print(NEW "tfsupport ALL=NOPASSWD: /usr/bin/tac\n");
			print(NEW "tfsupport ALL=NOPASSWD: /usr/bin/mdir\n");
			print(NEW "tfsupport ALL=NOPASSWD: /usr/bin/nmap\n");
			print(NEW "tfsupport ALL=NOPASSWD: /usr/sbin/up2date\n");
			print(NEW "tfsupport ALL=NOPASSWD: /usr/sbin/dmidecode\n");
			print(NEW "tfsupport ALL=NOPASSWD: /usr/sbin/aide\n");
			print(NEW "tfsupport ALL=NOPASSWD: /usr/sbin/sosreport\n");
			print(NEW "tfsupport ALL=NOPASSWD: /usr/sbin/eject\n");
			print(NEW "tfsupport ALL=NOPASSWD: /usr/sbin/system-config-network\n");
			print(NEW "tfsupport ALL=NOPASSWD: /usr/sbin/system-config-printer\n");
			print(NEW "tfsupport ALL=NOPASSWD: $RTIDIR/bin/rtiuser.pl\n");
			print(NEW "tfsupport ALL=NOPASSWD: $RTIDIR/bin/doveserver.pl\n");
			print(NEW "tfsupport ALL=NOPASSWD: $RTIDIR/bin/altiriscontrol.pl\n");
			print(NEW "tfsupport ALL=NOPASSWD: $RTIDIR/bin/rti\n");
			print(NEW "tfsupport ALL=NOPASSWD: $RTIDIR/bin/killem\n");
			print(NEW "tfsupport ALL=NOPASSWD: $RTIDIR/bin/startbbx\n");
			print(NEW "tfsupport ALL=NOPASSWD: $RTIDIR/bin/checkit\n");
			print(NEW "tfsupport ALL=NOPASSWD: $RTIDIR/bin/dosdir\n");
			print(NEW "tfsupport ALL=NOPASSWD: $RTIDIR/bin/rti_mtools_wrapper\n");
		}
	}
	close(NEW);
	close(OLD);

	return(0);
}
